{ RRRRRR                  ReportBuilder Class Library                  BBBBB
  RR   RR                                                              BB   BB
  RRRRRR                 Digital Metaphors Corporation                 BB BB
  RR  RR                                                               BB   BB
  RR   RR                   Copyright (c) 1996-2002                    BBBBB   }
{
Report Builder Dade UniDac Plug Ins( http://www.devart.com/unidac/ )
Based on the DOA Plug Ins
Written by Laurent NAVARRO ln@altidev.com
This Code is Totaly Free

V1.0 : 09/08/2009 : Initial version
V1.1 : 10/08/2009 :
  Added UnidadeTableListRestriction variable to restrict table browsing to a specific schema
V1.2 : 12/08/2009 :
  BugFix for MySQL
V1.3 : 11/10/2009 :
   Changed the behavior of the connection dialog.
   Created a sample application

Usage Note :
By default, when configuring Data Access, UniDade will propose to create a new connection
you can cancel, it will then just propose you the available connection components on the designer form/DM

If you want avoid the New Connection Creation you can use FUniDacConnection global variable
  FUniDacConnection := TUniConnection.Create(nil);
  FUniDacConnection.AssignConnect(YourConnectionComponent);

  I also suggest you this code to configure this connection by default:
   ppDesigner1.DataSettings.SessionType:='UniDacSession';
   ppDesigner1.DataSettings.DatabaseName:='YourConnectionComponent';
   ppDesigner1.DataSettings.DatabaseType :=TdaUniDacSession.GetCnxDatabaseType(YourConnectionComponent);

I know it's a bit complex, but I wasn't able to found a simpler solution which keep also the flexibility of creating a new connection of any type.

If you want restrict table browsing to only one schema you can variable UnidadeTableListRestriction global variable
  UnidadeTableListRestriction:='YourSchema';

This plug ins have been tested with Oracle 11G, MS SQL 2005, MySQL 5.31 

}
unit daUniDac;

interface

uses Classes, SysUtils, Forms, ExtCtrls,  DB, 
     ppClass, ppComm, ppDBPipe, ppDB, ppClasUt, ppTypes,
     daDB, daQueryDataView, daDataView, daPreviewDataDlg,
     UniProvider, SQLServerUniProvider, DBAccess, Uni,UniDacVcl;

type


  { TdaChildUniDacQuery }
  TdaChildUniDacQuery = class(TUniQuery)
    public
      function HasParent: Boolean; override;
    end;  {class, TdaChildUniDacQuery}

  { TdaUniDacSession }
  TdaUniDacSession = class(TdaSession)
    private
      FFunctions: TStringList;


    protected
      function GetDefaultDatabase(const aDatabaseName: String): TComponent; override;
      function IsNamedDatabase(const aDatabaseName: String; aDatabase: TComponent): Boolean; override;

    public
      constructor Create(aComponent: TComponent); override;
      destructor Destroy; override;

      class function ClassDescription: String; override;
      class function DataSetClass: TdaDataSetClass; override;
      class function DatabaseClass: TComponentClass; override;
      class function GetDefaultUniDacConnection: TUniConnection;
      class function LogonToDatabase(aUniDacConnection: TUniConnection): Boolean;
      class function GetCnxDatabaseType(aUniDacConnection: TUniConnection): TppDatabaseType;

      procedure GetDatabaseNames(aList: TStrings); override;
      function  GetDatabaseType(const aDatabaseName: String): TppDatabaseType; override;
      procedure GetTableNames(const aDatabaseName: String; aList: TStrings); override;
      function  ValidDatabaseTypes: TppDatabaseTypes; override;

  end; {class, TdaUniDacSession}



  { TdaUniDacDataSet }
  TdaUniDacDataSet = class(TdaDataSet)
    private
      FQuery: TUniQuery;

      function GetQuery: TUniQuery;

    protected
      procedure BuildFieldList; override;
      function  GetActive: Boolean; override;
      procedure SetActive(Value: Boolean); override;
      procedure SetDatabase(aDatabase: TComponent); override;
      procedure SetDataName(const aDataName: String); override;
      property Query: TUniQuery read GetQuery;

    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;

      class function ClassDescription: String; override;

      procedure GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings); override;
      procedure GetFieldsForSQL(aList: TList; aSQL: TStrings); override;

  end; {class, TdaUniDacDataSet}


  { TdaUniDacQueryDataView }
  TdaUniDacQueryDataView = class(TdaQueryDataView)
    private
      FDataSource: TppChildDataSource;
      FQuery: TdaChildUniDacQuery;

    protected
      procedure SQLChanged; override;

    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;

      class function PreviewFormClass: TFormClass; override;
      class function SessionClass: TClass; override;

      procedure Init; override;
      procedure ConnectPipelinesToData; override;

    published
      property DataSource: TppChildDataSource read FDataSource;

  end; {class, TdaUniDacQueryDataView}


  {Delphi design time registration}
  procedure Register;
var
  // This variable is global to allow to set it on your application session.
  // if you set it, it will use this connection
  FUniDacConnection: TUniConnection;
  // Allow to restrict list of table y specifying the schema.
  UnidadeTableListRestriction : string ='';

implementation

const
  cDefaultConnection = 'DefaultUniDacConnection';

var
  FUniDacLogon: TCustomConnectDialog;

{******************************************************************************
 *
 ** R E G I S T E R
 *
{******************************************************************************}

procedure Register;
begin

  {UniDac DataAccess Components}
  RegisterNoIcon([TdaChildUniDacQuery]);

  {UniDac DataViews}
  RegisterNoIcon([TdaUniDacQueryDataView]);

end;

{******************************************************************************
 *
 ** C H I L D   UNIDAC   D A T A   A C C E S S   C O M P O N E N T S
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaChildUniDacQuery.HasParent }

function TdaChildUniDacQuery.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}

{******************************************************************************
 *
 ** UniDac  S E S S I O N
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaUniDacSession.Create }

constructor TdaUniDacSession.Create(aComponent: TComponent);
begin

  inherited Create(aComponent);

  FFunctions := TStringList.Create;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaUniDacSession.Destroy }

destructor TdaUniDacSession.Destroy;
begin

  FFunctions.Free;

  inherited Destroy;

end; {destructor, Destroy}


{------------------------------------------------------------------------------}
{ TdaUniDacSession.ClassDescription }

class function TdaUniDacSession.ClassDescription: String;
begin
  Result := 'UniDacSession';
end; {class function, ClassDescription}

{------------------------------------------------------------------------------}
{ TdaUniDacSession.DataSetClass }

class function TdaUniDacSession.DataSetClass: TdaDataSetClass;
begin
  Result := TdaUniDacDataSet;
end; {class function, DataSetClass}

{------------------------------------------------------------------------------}
{ TdaUniDacSession.DatabaseClass }

class function TdaUniDacSession.DatabaseClass: TComponentClass;
begin
  Result := TUniConnection;
end;

{------------------------------------------------------------------------------}
{ TdaUniDacSession.GetDefaultUniDacConnection }

class function TdaUniDacSession.GetDefaultUniDacConnection: TUniConnection;
begin

  {create the default Connection, if needed}
  if (FUniDacConnection = nil) then
    begin
      {create default UniDac Connection}
      FUniDacConnection := TUniConnection.Create(nil);
      FUniDacConnection.Name := cDefaultConnection;
    end;
  { Manage the Connection dialog }
  if FUniDacConnection.ConnectDialog= nil then
    begin
      FUniDacLogon := TUniConnectDialog.Create(nil);
      FUniDacConnection.ConnectDialog:=FUniDacLogon;
    end
    else
      if FUniDacLogon=nil then
        FUniDacLogon:=FUniDacConnection.ConnectDialog;

  Result := FUniDacConnection;

end; {class function, GetDefaultUniDacConnection}

{------------------------------------------------------------------------------}
{ TdaUniDacSession.LogonToDatabase }

class function TdaUniDacSession.LogonToDatabase(aUniDacConnection: TUniConnection): Boolean;
begin

  if (aUniDacConnection = nil) then
    Result := False

  else if aUniDacConnection.Connected then
    Result := True

  else
    begin
      if (aUniDacConnection = GetDefaultUniDacConnection) then
        FUniDacLogon.Execute
      else
        aUniDacConnection.Connected := True;

      Result := aUniDacConnection.Connected;
    end;

end; {class function, LogonToDatabase}


class function TdaUniDacSession.GetCnxDatabaseType(aUniDacConnection: TUniConnection): TppDatabaseType;
begin
   if aUniDacConnection.ProviderName='SQL Server' then
    Result := dtMSSQLServer
   else if aUniDacConnection.ProviderName='Oracle' then
    Result := dtOracle
   else if aUniDacConnection.ProviderName='MySQL' then
    Result := dtMySQL
   else if aUniDacConnection.ProviderName='InterBase' then
    Result := dtInterBase
   else if aUniDacConnection.ProviderName='PostgreSQL' then
    Result := dtPostgreSQL
   else
    Result := dtOther;

end; {procedure, GetDatabaseType}

{------------------------------------------------------------------------------}
{ TdaUniDacSession.GetDatabaseType }
function TdaUniDacSession.GetDatabaseType(const aDatabaseName: String): TppDatabaseType;
var
  aUniDacConnection: TUniConnection;
begin
   aUniDacConnection:=GetDefaultUniDacConnection;
   result := GetCnxDatabaseType(aUniDacConnection);
end; {procedure, GetDatabaseType}

{------------------------------------------------------------------------------}
{ TdaUniDacSession.GetTableNames }

procedure TdaUniDacSession.GetTableNames(const aDatabaseName: String; aList: TStrings);
var
  lsTableName: String;
  lConnection: TUniConnection;
  lQuery: TUniMetaData;

begin
  aList.Clear;
  lConnection := TUniConnection(GetDatabaseForName(aDatabaseName));
  if not LogonToDatabase(lConnection) then Exit;
  {get list of table names from a table object}
  lQuery := TUniMetaData.Create(nil);
  lQuery.Connection := lConnection;
  lQuery.MetaDataKind:='Tables';
{  V1.2 don't restrict to table, view are also interesting
  if lConnection.ProviderName='MySQL' then
     lQuery.Restrictions.Values['TABLE_TYPE']:='BASE TABLE'
  else
     lQuery.Restrictions.Values['TABLE_TYPE']:='TABLE';
     }
  lQuery.Restrictions.Values['TABLE_SCHEMA']:=UnidadeTableListRestriction;


  lQuery.Open;

  if lQuery.Active then
    begin
      lQuery.First;
      while not lQuery.EOF do
        begin
          lsTableName := lQuery.FieldbyName('TABLE_SCHEMA').AsString + '.' + lQuery.FieldbyName('TABLE_NAME').AsString;
          aList.Add(lsTableName);
          lQuery.Next;
        end;
    end;
  lQuery.Free;

end; {procedure, GetTableNames}


{------------------------------------------------------------------------------}
{ TdaUniDacSession.GetDatabaseNames }

procedure TdaUniDacSession.GetDatabaseNames(aList: TStrings);
var
  lQuery: TUniMetaData;
begin
  {call inherited to build list of available TUniConnection components}
  inherited GetDatabaseNames(aList);

  if not LogonToDatabase(GetDefaultUniDacConnection) then Exit;

  if GetDefaultUniDacConnection.ProviderName='Oracle' then
    exit; // No Database concept under Oracle, only host 

  lQuery := TUniMetaData.Create(nil);
  lQuery.Connection := GetDefaultUniDacConnection;
  lQuery.MetaDataKind:='Databases';
  lQuery.Open;

  if lQuery.Active then
    begin
      lQuery.First;
      while not lQuery.EOF do
        begin
          aList.Add(lQuery.FieldbyName('DATABASE_NAME').AsString);
          lQuery.Next;
        end;
    end;
  lQuery.Free;
end; {procedure, GetDatabaseNames}

{------------------------------------------------------------------------------}
{ TdaUniDacSession.GetDefaultDatabase }

function TdaUniDacSession.GetDefaultDatabase(const aDatabaseName: String): TComponent;
var
  lUniSession: TUniConnection;
begin

  lUniSession := GetDefaultUniDacConnection;

  {set DatabaseName property, if needed}
  if (lUniSession.Database <> aDatabaseName) then
    begin
      if lUniSession.Connected then
        lUniSession.Connected := False;
      lUniSession.Database := aDatabaseName;

      {UniDac does not auto logon, so force a logon dialog here}
      LogonToDatabase(lUniSession);

    end;

  Result := lUniSession;

end; {function, GetDefaultDatabase}

{------------------------------------------------------------------------------}
{ TdaUniDacSession.IsNamedDatabase }

function TdaUniDacSession.IsNamedDatabase(const aDatabaseName: String; aDatabase: TComponent): Boolean;
begin

   Result := (AnsiCompareStr(aDatabase.Name, aDatabaseName) = 0) or
             (AnsiCompareStr(TUniConnection(aDatabase).Database, aDatabaseName) = 0);

end; {function, IsNamedDatabase}

{------------------------------------------------------------------------------}
{ TdaUniDacSession.ValidDatabaseTypes }

function TdaUniDacSession.ValidDatabaseTypes: TppDatabaseTypes;
begin
  Result := [dtMySQL,dtMSSQLServer,dtInterBase,dtOracle,dtPostgreSQL,dtOther];
end; {function, ValidDatabaseTypes}


{******************************************************************************
 *
 ** UniDac D A T A S E T
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaUniDacDataSet.Create }

constructor TdaUniDacDataSet.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  FQuery := nil;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaUniDacDataSet.Destroy }

destructor TdaUniDacDataSet.Destroy;
begin

  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdaUniDacDataSet.ClassDescription }

class function TdaUniDacDataSet.ClassDescription: String;
begin
  Result := 'UniDacDataSet';
end; {class function, ClassDescription}

{------------------------------------------------------------------------------}
{ TdaUniDacDataSet.GetActive }

function TdaUniDacDataSet.GetActive: Boolean;
begin
  Result := GetQuery.Active
end; {function, GetActive}

{------------------------------------------------------------------------------}
{ TdaUniDacDataSet.SetActive }

procedure TdaUniDacDataSet.SetActive(Value: Boolean);
begin
  GetQuery.Active := Value;
end; {procedure, SetActive}

{------------------------------------------------------------------------------}
{ TdaUniDacDataSet.GetQuery }

function TdaUniDacDataSet.GetQuery: TUniQuery;
begin

  {create UniDac DataSet, if needed}
  if (FQuery = nil) then
    FQuery := TUniQuery.Create(Self);

  Result := FQuery;

end; {procedure, GetQuery}

{------------------------------------------------------------------------------}
{ TdaUniDacDataSet.SetDatabase }

procedure TdaUniDacDataSet.SetDatabase(aDatabase: TComponent);
begin

  inherited SetDatabase(aDatabase);

  {table cannot be active to set database property}
  if GetQuery.Active then
    FQuery.Active := False;

  {set ADOQuery.Connection}
  FQuery.Connection := TUniConnection(aDatabase);

end; {procedure, SetDatabase}

{------------------------------------------------------------------------------}
{ TdaUniDacDataSet.SetDataName }

procedure TdaUniDacDataSet.SetDataName(const aDataName: String);
begin

  inherited SetDataName(aDataName);

  {dataset cannot be active to set data name}
  if GetQuery.Active then
    FQuery.Active := False;

  {construct an SQL statment that returns an empty result set,
   this is used to get the field information }
  FQuery.SQL.Text := 'SELECT * FROM ' + aDataName +
                     ' WHERE ''c'' <> ''c'' ';

end; {procedure, SetDataName}

{------------------------------------------------------------------------------}
{ TdaUniDacDataSet.BuildFieldList }

procedure TdaUniDacDataSet.BuildFieldList;
var
  liIndex: Integer;
  lQueryField: TField;
  lField: TppField;
begin

  inherited BuildFieldList;


  {set table to active}
  if not(GetQuery.Active) then
    FQuery.Active := True;

  {create TppField objects for each field in the table}
  for liIndex := 0 to FQuery.FieldCount - 1 do
    begin
      lQueryField := FQuery.Fields[liIndex];

      lField := TppField.Create(nil);

      lField.TableName := DataName;
      lField.FieldName := lQueryField.FieldName;
      lField.FieldAlias := lQueryField.DisplayLabel;
      lField.FieldLength := lQueryField.Size;
      lField.DataType := ppConvertFieldType(lQueryField.DataType);
      lField.DisplayWidth := lQueryField.DisplayWidth;


      AddField(lField);
    end;

end; {function, BuildFieldList}

{------------------------------------------------------------------------------}
{ TdaUniDacDataSet.GetFieldNamesForSQL }

procedure TdaUniDacDataSet.GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings);
var
  lQuery: TUniQuery;
begin
  aList.Clear;

  {create a temporary query}
  lQuery := TUniQuery.Create(Self);

  {set the database and SQL properties}
  lQuery.Connection := TUniConnection(Database);
  lQuery.SQL := aSQL;

  {get the field names}
  lQuery.GetFieldNames(aList);

  lQuery.Free;

end; {procedure, GetFieldNamesForSQL}

{------------------------------------------------------------------------------}
{ TdaUniDacDataSet.GetFieldsForSQL }

procedure TdaUniDacDataSet.GetFieldsForSQL(aList: TList; aSQL: TStrings);
var
  lQuery: TUniQuery;
  lQueryField: TField;
  lField: TppField;
  liIndex: Integer;
begin
  aList.Clear;

  {create a temporary query}
  lQuery := TUniQuery.Create(Self);

  try
    {assign databae and SQL properties}
    lQuery.Connection := TUniConnection(Database);
    lQuery.SQL := aSQL;

    {set query to active}
    lQuery.Active := True;

    {create a TppField object for each field in the query}
    for liIndex := 0 to lQuery.FieldCount - 1 do
      begin
        lQueryField := lQuery.Fields[liIndex];

        lField := TppField.Create(nil);

        lField.FieldName := lQueryField.FieldName;
        lField.FieldAlias := lQueryField.DisplayLabel;
        lField.FieldLength := lQueryField.Size;
        lField.DataType := ppConvertFieldType(lQueryField.DataType);
        lField.DisplayWidth := lQueryField.DisplayWidth;

        aList.Add(lField);
      end;

  finally
    lQuery.Free;
    
  end;

end; {procedure, GetFieldsForSQL}


{******************************************************************************
 *
 ** UniDac  Q U E R Y   D A T A V I E W
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaUniDacQueryDataView.Create }

constructor TdaUniDacQueryDataView.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  {notes: 1. must use ChildQuery, ChildDataSource, ChildPipeline etc.
          2. use Self as owner for Query, DataSource etc.
          3. do NOT assign a Name }

  FQuery := TdaChildUniDacQuery.Create(Self);

  FDataSource := TppChildDataSource.Create(Self);
  FDataSource.DataSet := FQuery;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaUniDacQueryDataView.Destroy }

destructor TdaUniDacQueryDataView.Destroy;
begin
  FDataSource.Free;
  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdaUniDacQueryDataView.PreviewFormClass }

class function TdaUniDacQueryDataView.PreviewFormClass: TFormClass;
begin
  Result := TFormClass(GetClass('TdaPreviewDataDialog'));
end; {class function, PreviewFormClass}

{------------------------------------------------------------------------------}
{ TdaUniDacQueryDataView.ConnectionClass }

class function TdaUniDacQueryDataView.SessionClass: TClass;
begin
  Result := TdaUniDacSession;
end; {class function, SessionClass}

{------------------------------------------------------------------------------}
{ TdaUniDacQueryDataView.ConnectPipelinesToData }

procedure TdaUniDacQueryDataView.ConnectPipelinesToData;
begin

  if DataPipelineCount = 0 then Exit;

  {need to reconnect here}
  TppDBPipeline(DataPipelines[0]).DataSource := FDataSource;

end; {procedure, ConnectPipelinesToData}

{------------------------------------------------------------------------------}
{ TdaUniDacQueryDataView.Init }

procedure TdaUniDacQueryDataView.Init;
var
  lDataPipeline: TppChildDBPipeline;

begin

  inherited Init;

  if DataPipelineCount > 0 then Exit;

  {note: DataView's owner must own the DataPipeline }
  lDataPipeline := TppChildDBPipeline(ppComponentCreate(Self, TppChildDBPipeline));
  lDataPipeline.DataSource := FDataSource;
 
  lDataPipeline.AutoCreateFields := False;

  {add DataPipeline to the dataview }
  lDataPipeline.DataView := Self;

end; {procedure, Init}

{------------------------------------------------------------------------------}
{ TdaUniDacQueryDataView.SQLChanged }

procedure TdaUniDacQueryDataView.SQLChanged;
var
  lDatabase: TComponent;
begin

  if FQuery.Active then
    FQuery.Close;

  lDatabase := Session.GetDatabaseForName(SQL.DatabaseName);

  FQuery.Connection := TUniConnection(lDatabase);
  FQuery.SQL := SQL.MagicSQLText;

end; {procedure, SQLChanged}



initialization

  {register the UniDac descendant classes}
  RegisterClasses([TdaChildUniDacQuery]);

  {register DADE descendant session, dataset, dataview}
  daRegisterSession(TdaUniDacSession);
  daRegisterDataSet(TdaUniDacDataSet);
  daRegisterDataView(TdaUniDacQueryDataView);

  {initialize internal reference variables}
  FUniDacConnection     := nil;
  FUniDacLogon          := nil;


finalization

  {free the default connection object}
  FUniDacConnection.Free;
  FUniDacConnection := nil;
  FUniDacLogon.Free;
  FUniDacLogon := nil;

  {unregister the UniDac descendant classes}
  UnRegisterClasses([TdaChildUniDacQuery]);

  {unregister DADE descendant the session, dataset, dataview}
  daUnRegisterSession(TdaUniDacSession);
  daUnRegisterDataSet(TdaUniDacDataSet);
  daUnRegisterDataView(TdaUniDacQueryDataView);


end.
